;**************************************************************************
;*
;* FILE  intro.asm
;* Copyright (c) 2007 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: intro.asm,v 1.14 2007-12-07 16:15:47 tlr Exp $
;*
;* DESCRIPTION
;*   Intro to statistical cracker
;*
;******
	processor 6502

	include	"macros.i"
	
INTRO_START	equ	$2000
DEBUG	equ	0

BG_COLOR	equ	11
	
	seg.u	zp
;**************************************************************************
;*
;* SECTION  zero page
;*
;******
	org	$f7
tmp_zp:
	ds.b	1
tmp2_zp:
	ds.b	1
cnt_zp:
	ds.b	1
ptr_zp:
	ds.w	1
cmp_zp:
	ds.b	1

	seg	code
;**************************************************************************
;*
;* SECTION  code
;*
;******
	org	INTRO_START
	mac	DEBUG12
	if	DEBUG
	inc	$d020
	dec	$d020
	else
	jsr	twelve
	endif
	endm

;**************************************************************************
;*
;* SECTION  intro page
;*   
;******
intro:
	sei
	jsr	convert_cset
	jsr	init_colors

; wait for vertical blanking and disable screen
	jsr	wait_vb
	jsr	$e5a0
	lda	#$0b
	sta	$d011
	sta	$d020
	lda	#$1e
	sta	$d018

	jsr	print	

	if	DEBUG
	inc	$d020
	endif

; wait for vertical blanking and enable screen again.
	jsr	wait_vb
	lda	#$1b
	sta	$d011

	lda	#3
	sta	cmp_zp

LINES	equ	200

;*** main loop ***
int_lp1:
	jsr	stabilize
	jsr	twelve
	ldx	#0
	START_SAMEPAGE "colorloop"
int_lp2:
	lda	ctab3,x		; 4
	sta	ctab3-2,x	; 5
int_sm1:
	ora	ctab1,x		; 4
	ora	ctab2,x		; 4   
	tay			; 2
	lda	look,y		; 4
	sta	int_sm3+1	; 4	=27
	inx			; 2
	lda	ctab3,x		; 4
	sta	ctab3-2,x	; 5
int_sm2:
	ora	ctab1,x		; 4
	ora	ctab2,x		; 4   
	tay			; 2
	lda	look,y		; 4	=25 +27= 52
int_sm3:
	ldy	#0		; 2
	sty	$d021		; 4	=6 + 52 =58
	tay			; 2	
	lda	$d012		; 4
	and	#$7		; 2
	cmp	cmp_zp		; 3
	beq	int_skp1	; 2(3)	=13(14)
	jsr	twelve		; 12
	jsr	twelve		; 12
	jsr	twelve		; 12
	nop			; 2
	nop			; 2
	nop			; 2
	nop			; 2	=44 +13=57
int_skp1:
	inx			; 2
	sty	$d021		; 4	=6 +57(14)=63(20)
	cpx	#LINES		; 2
	bne	int_lp2		; 3	=5 +58=63
	END_SAMEPAGE

	ldx	#LINES-1
int_lp3:
	lda	ctab2-1,x
	sta	ctab2,x
	dex
	bne	int_lp3

	ldx	#1
	jsr	sequencer
	sta	ctab2
	ldx	#2
	jsr	sequencer
	sta	ctab3+LINES-2
	jsr	sequencer
	sta	ctab3+LINES-1

	inc	half_cnt
	lda	half_cnt
	lsr
	bcc	int_skp3
	ldy	int_sm1+1
	iny
	cpy	#<[ctab1+$10]
	bne	int_skp2
	ldy	#<ctab1
int_skp2:
	sty	int_sm1+1
	sty	int_sm2+1
int_skp3:

	lda	#$7f
	sta	$dc00
	lda	$dc01
	and	#%00010000
	beq	int_ex1
	jmp	int_lp1
int_ex1:
	jsr	wait_vb
	lda	#$0b
	sta	$d011
	cli
	rts

;**************************************************************************
;*
;* NAME  stabilize
;*
;* DESCRIPTION
;*   Find a stable point at vertical blanking
;*
;******
stabilize:
; wait until positive part
stb_lp1:
	lda	$d011
	bmi	stb_lp1

; find decent position on raster line
	ldx	$d012
	inx
stb_lp2:
	cpx	$d012
	bne	stb_lp2

; wait until positive part

 	jsr	twelve		
	lda	#$31
	sta	tmp_zp
;*** do stabilization ***
	ldx	$d012
	START_SAMEPAGE "stabilize"
stb_lp3:
	jsr	twelve	;12
	DEBUG12		;12
	jsr	twelve	;12
	jsr	twelve	;12	=48
	cpx	$d012	;4
	beq	stb_skp1 ;2(3)
stb_skp1:
	inx		;2
	cpx	tmp_zp	;3
	bne	stb_lp3	;3	=14(15) +48=62(63)
	END_SAMEPAGE
;*** now we are really stable ***
twelve:
	rts

;**************************************************************************
;*
;* NAME  wait_vb
;*
;* DESCRIPTION
;*   Wait for vertical blanking
;*
;******
wait_vb:
wv_lp1:
	lda	$d011
	bpl	wv_lp1
wv_lp2:
	lda	$d011
	bmi	wv_lp2
	rts

;**************************************************************************
;*
;* NAME  sequencer
;*
;* DESCRIPTION
;*   Color table sequencer
;*   in: X=table num
;*   out: Acc=value
;*
;******
sequencer:
	dec	cseq_cnt,x
	bne	sq_skp2
	ldy	cseq_pos,x
	jmp	sq_skp1
sq_lp1:
	ldy	cseq_restart,x
sq_skp1:
	lda	cseq,y
	beq	sq_lp1
	sta	cseq_cnt,x
	lda	cseq+1,y
	sta	cseq_data,x
	iny
	iny
	tya
	sta	cseq_pos,x
sq_skp2:
	lda	cseq_data,x
	rts

cseq_pos:
	dc.b	cseq1,cseq2,cseq3
cseq_cnt:
	dc.b	1,1,1
cseq_restart:
	dc.b	cseq1_rst,cseq2_rst,cseq3_rst
cseq_data:
	dc.b	0,0,0

;*** Color data ***
cseq:
cseq1	equ	.-cseq		; speed 0.5
cseq1_rst equ	.-cseq
	dc.b	4,1
	dc.b	12,0
	dc.b	0
cseq2	equ	.-cseq		; speed 1
	dc.b	100,0
cseq2_rst equ	.-cseq
	dc.b	80,2
	dc.b	160,0
	dc.b	80,4
	dc.b	160,0
	dc.b	80,6
	dc.b	160,0
	dc.b	0
cseq3	equ	.-cseq		; speed 2
	dc.b	200,0
cseq3_rst equ	.-cseq
	dc.b	16,8
	dc.b	200,0
	dc.b	200,0
	dc.b	0


	START_SAMEPAGE "lookup"
;*** color look up table ***
look:
	dc.b	5,13,5,7,5,3,5,15
	dc.b	13,1,5,7,5,3,5,15
	END_SAMEPAGE
	
;**************************************************************************
;*
;* NAME  init_colors
;*
;* DESCRIPTION
;*   initialize color tables
;*
;******
init_colors:
	ldx	#0
ic_lp1:
	jsr	sequencer
ic_sm1:
	ldy	#0
	sta	ctab1,y
	lda	#0
	sta	ctab2,y
	sta	ctab3,y
	inc	ic_sm1+1
	bne	ic_lp1
	rts

;**************************************************************************
;*
;* NAME  convert_cset
;*
;* DESCRIPTION
;*   Create 2x2 charset.
;*
;******
SOURCE_CSET	equ	$d400
CHARSET		equ	$3800
convert_cset:
	lda	#$33
	sta	$01
	lda	#2
	sta	cnt_zp
	ldy	#0
cc_lp1:
cc_sm1:
	lda	SOURCE_CSET,y
	sta	tmp_zp
	jsr	cc_rot
cc_sm2:
	sta	CHARSET,x
	sta	CHARSET+1,x
	jsr	cc_rot
cc_sm3:
	sta	CHARSET+$400,x
	sta	CHARSET+$401,x
	inx
	inx
	bne	cc_skp1
	inc	cc_sm2+2
	inc	cc_sm2+5
	inc	cc_sm3+2
	inc	cc_sm3+5
cc_skp1:
	iny
	bne	cc_lp1
	inc	cc_sm1+2
	dec	cnt_zp
	bne	cc_lp1
	lda	#$37
	sta	$01
	rts
cc_rot:
	asl	tmp_zp
	rol
	asl
	asl	tmp_zp
	rol
	asl
	asl	tmp_zp
	rol
	asl
	asl	tmp_zp
	rol
	asl
	sta	tmp2_zp
	lsr
	ora	tmp2_zp
	rts

;**************************************************************************
;*
;* NAME  print
;*
;* DESCRIPTION
;*   Print screen contents
;*
;******
print:
	ldx	#0
prt_lp1:
	lda	#$40
	sta	$0400,x
	sta	$0500,x
	sta	$0600,x
	sta	$06e8,x
	lda	#BG_COLOR
	sta	$d800,x
	sta	$d900,x
	sta	$da00,x
	sta	$dae8,x
	inx
	bne	prt_lp1

prt_lp2:
	jsr	get_byte
	sta	ptr_zp
	jsr	get_byte
	sta	ptr_zp+1
	ora	ptr_zp
	beq	prt_ex1

	ldy	#0
	sty	cnt_zp
prt_lp3:
	ldy	cnt_zp
	jsr	get_byte
	and	#$3f
	beq	prt_lp2
	asl
	tax
	sta	(ptr_zp),y
	ora	#$80
	iny
	sta	(ptr_zp),y
	tya
	clc
	adc	#40-1
	tay		
	inx
	txa
	sta	(ptr_zp),y
	ora	#$80
	iny
	sta	(ptr_zp),y
	inc	cnt_zp
	inc	cnt_zp
	bne	prt_lp3		; always taken
prt_ex1:
	rts

get_byte:
gb_sm1:
	lda	screen	
	inc	gb_sm1+1
	bne	gb_ex1
	inc	gb_sm1+2
gb_ex1:
	rts

	mac	text
	dc.w	$0400+40*{2}+{1}
	dc.b	{3}
	dc.b	0
	endm
screen:
	text	8,0,    "> DE-CODER <"
	text	1,2,  "STATISTICAL CRACKER"
	text	7,6,  "AUTO DECODES:"
	text	9,9,     "*TCD CODER*"
	text	6,11,    "*ONEWAY CODER*"
	text	3,13,   "*SNACKY/GP CODER*"
	text	2,15,   "*TIME CODER V1/V2*"
	text	9,19,     "PRESS SPACE"
	text	0,23, "PROGRAMMED BY TLR'07"
	dc.w	0

end_code	equ	.
	seg.u	bss
;**************************************************************************
;*
;* SECTION  bss
;*   
;******
	org	end_code

half_cnt:
	ds.b	1
	
	align	256
	ds.b	16
ctab1:
	ds.b	LINES
	align	256
	ds.b	16
ctab2:
	ds.b	LINES
	align	256
	ds.b	16
ctab3:
	ds.b	LINES
; eof
