;**************************************************************************
;*
;* FILE  macros.i
;* Copyright (c) 1995-1996, 2002, 2007 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: macros.i,v 1.2 2007-12-07 16:24:48 tlr Exp $
;*
;* DESCRIPTION
;*   Useful macros 
;*
;******
	IFNCONST MACROS_I
MACROS_I	EQU	1

;**************************************************************************
;*
;* NAME  INFO, WARNING, ERROR
;*
;* SYNOPSIS
;*   INFO <str>,<str>,...
;*   WARNING <str>,<str>,...
;*   ERROR <str>,<str>,...
;*
;* DESCRIPTION
;*   info,warning & error macros.
;*
;* KNOWN BUGS
;*   none
;* 
;******
	MAC	INFO
	echo	"INFO:", {0}
	ENDM
	MAC	ERROR
	echo	"ERROR!:", {0}
	ERR
	ENDM
	MAC	WARNING
	echo	"WARNING!:", {0}
	ENDM

;**************************************************************************
;*
;* NAME  START_SAMEPAGE, END_SAMEPAGE
;*
;* SYNOPSIS
;*   START_SAMEPAGE [<label>]
;*   END_SAMEPAGE
;* 
;* DESCRIPTION
;*   These macros marks a section that must stay within the same page.
;*   If the condition is violated, assembly will be aborted.
;*   An optional label argument may be given to START_SAMEPAGE.  This
;*   label is output when showing the address range of the section.
;*
;* KNOWN BUGS
;*   none
;* 
;******
	MAC	START_SAMEPAGE
_SAMEPAGE_MARKER	SET	.
_SAMEPAGE_NAME		SET	{0}
	ENDM

	MAC	END_SAMEPAGE
	IF	>_SAMEPAGE_MARKER != >.
	IF	_SAMEPAGE_NAME!=""
	ERROR	"[SAMEPAGE] Page crossing not allowed! (",_SAMEPAGE_NAME,"@",_SAMEPAGE_MARKER,"-",.,")"
	ELSE
	ERROR	"[SAMEPAGE] Page crossing not allowed! (",_SAMEPAGE_MARKER,"-",.,")"
	ENDIF	
	ELSE
	IF	_SAMEPAGE_NAME!=""
	INFO	"[SAMEPAGE] successful SAMEPAGE. (",_SAMEPAGE_NAME,"@",_SAMEPAGE_MARKER,"-",.,")"
	ELSE
	INFO	"[SAMEPAGE] successful SAMEPAGE. (",_SAMEPAGE_MARKER,"-",.,")"
	ENDIF
	ENDIF
	ENDM

;**************************************************************************
;*
;* NAME  SHOWRANGE
;*
;* SYNOPSIS
;*   SHOWRANGE <str>,<startaddr>,<endaddr>
;*
;* DESCRIPTION
;*   print out a string followed by start, end, len, and len in blks
;*
;* KNOWN BUGS
;*   none
;* 
;******
	MAC	SHOWRANGE
	INFO	"[SHOWRANGE]",{1},[{2}],"-",[{3}],"(=",[{3}]-[{2}],"bytes,",([{3}]-[{2}]+253+2)/254,"blocks)"
	ENDM


;**************************************************************************
;*
;* NAME  ALIGN2
;*
;* SYNOPSIS
;*   ALIGN2 <str>,<align>
;*
;* DESCRIPTION
;*   print out a string followed by start, end, len, and len in blks
;*
;* KNOWN BUGS
;*   none
;* 
;******
	MAC	ALIGN2
_align	set	.
	align	{2}
	INFO	"[ALIGN2] align waste",.-_align,"bytes (",{1},")"
	ENDM

;**************************************************************************
;*
;* NAME  DECSTR
;*
;* SYNOPSIS
;*   DECSTR <value>
;*
;* DESCRIPTION
;*   make a string that is equal to <value>
;*
;* KNOWN BUGS
;*   none
;* 
;******
	MAC	DECSTR
_izero	SET	1	;initial zero
_div	SET	10000	;initial divisor
	REPEAT	5
_tmp	SET	[[{1}]/_div]%10
	IF	!_izero || _tmp!=0
	dc.b	"0"+_tmp
_izero	SET	0
	ENDIF
_div	SET	_div/10	
	REPEND
	ENDM

;**************************************************************************
;*
;* NAME  HEXDIGIT
;*
;* SYNOPSIS
;*   HEXDIGIT <value>
;*
;* DESCRIPTION
;*   make a single hex digit that is equal to <value>
;*
;* KNOWN BUGS
;*   none
;* 
;******
	MAC	HEXDIGIT
	IFCONST [{1}]
	IF	[{1}]<10
	dc.b	"0"+[{1}]
	ELSE
	dc.b	"A"+[{1}]-10
	ENDIF
	ELSE
	dc.b	"x"
	ENDIF
	ENDM

;**************************************************************************
;*
;* NAME  HEXWORD
;*
;* SYNOPSIS
;*   HEXDIGIT <value>
;*
;* DESCRIPTION
;*   make a string that is the hex value of <value>
;*
;* KNOWN BUGS
;*   none
;* 
;******
	MAC	HEXWORD
	HEXDIGIT [{1}]>>12
	HEXDIGIT [[{1}]>>8]&$0f
	HEXDIGIT [[{1}]>>4]&$0f
	HEXDIGIT [{1}]&$0f
	ENDM



	ENDIF ;MACROS.I
; eof

